﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;
using UnityEngine.EventSystems;

public class CharacterBehavior : MoverBehavior, ISubject<CharacterEventArgs>
{
    public int MaxHealth = 5;
    public int Health { get; private set; }
    public float LineOfSight = 3.0f;

    public bool alive { get; private set; }

	public GameObject hurtEffect;
	public GameObject healEffect;
    
	private AmebaAnimBehavior _anim;
    public AmebaAnimBehavior Anim
    {
        get
        {
            if (_anim == null)
                _anim = GetComponentInChildren<AmebaAnimBehavior>();

            return _anim;
        }
    }
    
    #region monobehavior
    
    public void Reset()
    {
        alive = true;
        Health = MaxHealth;

        if (Events != null)
            Events.Invoke(this, new CharacterEventArgs(CharacterEventType.Heal));

        ChangeState(new CharacterIdleState());
        transform.localScale = Vector3.one;
        RB.mass = 1;
        RB.velocity = Vector2.zero;
        RB.angularVelocity = 0.0f;
        transform.rotation = Quaternion.identity;
    }


    void TakeDamage(int damage)
    {
        if (Health == 0)
            return;

        if (Anim)
            Anim.TakingDamage();

        Health = Mathf.Clamp(Health - damage, 0, 10);
		Instantiate(hurtEffect,gameObject.transform.position, Quaternion.identity);

        //Events
        if(Health > 0)
        {
            if (Events != null)
                Events.Invoke(this, new CharacterEventArgs(CharacterEventType.TakeDamage));
        }
        else
        {
            if (Events != null)
                Events.Invoke(this, new CharacterEventArgs(CharacterEventType.Died));

            ChangeState(new CharacterDeadState());
        }
    }

    public void Heal(int count)
    {
		Instantiate(healEffect,gameObject.transform.position, Quaternion.identity);

        if(Health < MaxHealth)
        {
            Health = Mathf.Clamp(Health + count, 0, MaxHealth);
            
            if (Events != null)
                Events.Invoke(this, new CharacterEventArgs(CharacterEventType.Heal));

        }

        iTween.PunchScale(gameObject, Vector3.right * 0.5f, 0.6f);
    }

    void OnTriggerEnter2D(Collider2D other)
    {
        Transform cRoot = other.transform.root;
        switch (cRoot.tag)
        {
            case "Enemy":
                {
                    //TODO: move destroy logic to enemy behavior and perhaps ad some additional effects.
                    Vector2 dir = (transform.position - other.transform.position).normalized;
                    RB.AddForce(dir * 5, ForceMode2D.Impulse);
                    
                    EnemyBehavior eb = cRoot.GetComponent<EnemyBehavior>();
                    if(eb != null)
                    {
                        if (eb.DieOnHit)
                            eb.SB.Despawn();
                        
                        TakeDamage(eb.Damage);

                        int rNum = UnityEngine.Random.Range(0, 10);
                        if (rNum > 5)
                            RB.AddTorque(2.0f * eb.Damage );
                        else
                            RB.AddTorque(2.0f * - eb.Damage);
                    }
                }
                break;
            case "Food":
                {
                    FoodBehavior fb = cRoot.GetComponent<FoodBehavior>();
                    if(fb)
                    {
                        Heal(fb.points);
                        fb.SB.Despawn();
                        transform.localScale += Vector3.one * 0.2f;
                        RB.mass += 0.1f;
                    }
                }
                break;            
            default:
                break;
        }
    }

    BaseState<CharacterBehavior> _currentState;
    public void ChangeState(BaseState<CharacterBehavior> newState)
    {
        if (_currentState != null)
            _currentState.Exit(this);

        _currentState = newState;
        _currentState.Enter(this);
    }

    public virtual void Update()
    {
        if (_currentState != null)
            _currentState.Update(this);
    }

    protected override void FixedUpdate()
    {
        base.FixedUpdate();

        if (_currentState != null)
            _currentState.FixedUpdate(this);
    }

    public event EventHandler<CharacterEventArgs> Events;
    public void Attach(IObserver<CharacterEventArgs> observer)
    {
        Events += observer.HandleEvent;
    }

    public void Detach(IObserver<CharacterEventArgs> observer)
    {
        Events -= observer.HandleEvent;
    }

    #endregion
}



public enum CharacterEventType
{
    TakeDamage, Died, Heal
}

public class CharacterEventArgs : EventArgs
{
    public CharacterEventType EventType { get; private set; }
    public CharacterEventArgs(CharacterEventType evnt)
    {
        EventType = evnt;
    }
}